#ifndef __AUTH_NTLMSSP_PROTO_H__
#define __AUTH_NTLMSSP_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from auth/ntlmssp/ntlmssp.c  */


/**
 * Print out the NTLMSSP flags for debugging 
 * @param neg_flags The flags from the packet
 */
void debug_ntlmssp_flags(uint32_t neg_flags);

/**
 * Return the NTLMSSP master session key
 * 
 * @param gensec_ntlmssp_state NTLMSSP State
 */
NTSTATUS gensec_ntlmssp_session_key(struct gensec_security *gensec_security, 
				    DATA_BLOB *session_key);
void ntlmssp_handle_neg_flags(struct gensec_ntlmssp_state *gensec_ntlmssp_state,
			      uint32_t neg_flags, bool allow_lm);

/**
   Weaken NTLMSSP keys to cope with down-level clients and servers.

   We probably should have some parameters to control this, but as
   it only occours for LM_KEY connections, and this is controlled
   by the client lanman auth/lanman auth parameters, it isn't too bad.
*/
DATA_BLOB ntlmssp_weakend_key(struct gensec_ntlmssp_state *gensec_ntlmssp_state, 
			      TALLOC_CTX *mem_ctx) ;
NTSTATUS gensec_ntlmssp_start(struct gensec_security *gensec_security);

/* The following definitions come from auth/ntlmssp/ntlmssp_sign.c  */


/**
 * Some notes on the NTLM2 code:
 *
 * NTLM2 is a AEAD system.  This means that the data encrypted is not
 * all the data that is signed.  In DCE-RPC case, the headers of the
 * DCE-RPC packets are also signed.  This prevents some of the
 * fun-and-games one might have by changing them.
 *
 */
NTSTATUS gensec_ntlmssp_sign_packet(struct gensec_security *gensec_security, 
				    TALLOC_CTX *sig_mem_ctx, 
				    const uint8_t *data, size_t length, 
				    const uint8_t *whole_pdu, size_t pdu_length, 
				    DATA_BLOB *sig);

/**
 * Check the signature of an incoming packet 
 *
 */
NTSTATUS gensec_ntlmssp_check_packet(struct gensec_security *gensec_security, 
				     TALLOC_CTX *sig_mem_ctx, 
				     const uint8_t *data, size_t length, 
				     const uint8_t *whole_pdu, size_t pdu_length, 
				     const DATA_BLOB *sig);

/**
 * Seal data with the NTLMSSP algorithm
 *
 */
NTSTATUS gensec_ntlmssp_seal_packet(struct gensec_security *gensec_security, 
				    TALLOC_CTX *sig_mem_ctx, 
				    uint8_t *data, size_t length, 
				    const uint8_t *whole_pdu, size_t pdu_length, 
				    DATA_BLOB *sig);

/**
 * Unseal data with the NTLMSSP algorithm
 *
 */
NTSTATUS gensec_ntlmssp_unseal_packet(struct gensec_security *gensec_security, 
				      TALLOC_CTX *sig_mem_ctx, 
				      uint8_t *data, size_t length, 
				      const uint8_t *whole_pdu, size_t pdu_length, 
				      const DATA_BLOB *sig);

/**
   Initialise the state for NTLMSSP signing.
*/
NTSTATUS ntlmssp_sign_init(struct gensec_ntlmssp_state *gensec_ntlmssp_state);
size_t gensec_ntlmssp_sig_size(struct gensec_security *gensec_security, size_t data_size) ;
NTSTATUS gensec_ntlmssp_wrap(struct gensec_security *gensec_security, 
			     TALLOC_CTX *sig_mem_ctx, 
			     const DATA_BLOB *in, 
			     DATA_BLOB *out);
NTSTATUS gensec_ntlmssp_unwrap(struct gensec_security *gensec_security, 
			       TALLOC_CTX *sig_mem_ctx, 
			       const DATA_BLOB *in, 
			       DATA_BLOB *out);

/* The following definitions come from auth/ntlmssp/ntlmssp_client.c  */


/**
 * Next state function for the Initial packet
 * 
 * @param ntlmssp_state NTLMSSP State
 * @param out_mem_ctx The DATA_BLOB *out will be allocated on this context
 * @param in A NULL data blob (input ignored)
 * @param out The initial negotiate request to the server, as an talloc()ed DATA_BLOB, on out_mem_ctx
 * @return Errors or NT_STATUS_OK. 
 */
NTSTATUS ntlmssp_client_initial(struct gensec_security *gensec_security, 
				TALLOC_CTX *out_mem_ctx, 
				DATA_BLOB in, DATA_BLOB *out) ;

/**
 * Next state function for the Challenge Packet.  Generate an auth packet.
 * 
 * @param gensec_security GENSEC state
 * @param out_mem_ctx Memory context for *out
 * @param in The server challnege, as a DATA_BLOB.  reply.data must be NULL
 * @param out The next request (auth packet) to the server, as an allocated DATA_BLOB, on the out_mem_ctx context
 * @return Errors or NT_STATUS_OK. 
 */
NTSTATUS ntlmssp_client_challenge(struct gensec_security *gensec_security, 
				  TALLOC_CTX *out_mem_ctx,
				  const DATA_BLOB in, DATA_BLOB *out) ;
NTSTATUS gensec_ntlmssp_client_start(struct gensec_security *gensec_security);

/* The following definitions come from auth/ntlmssp/ntlmssp_server.c  */


/**
 * Next state function for the Negotiate packet
 * 
 * @param gensec_security GENSEC state
 * @param out_mem_ctx Memory context for *out
 * @param in The request, as a DATA_BLOB.  reply.data must be NULL
 * @param out The reply, as an allocated DATA_BLOB, caller to free.
 * @return Errors or MORE_PROCESSING_REQUIRED if (normal) a reply is required. 
 */
NTSTATUS ntlmssp_server_negotiate(struct gensec_security *gensec_security, 
				  TALLOC_CTX *out_mem_ctx, 
				  const DATA_BLOB in, DATA_BLOB *out) ;

/**
 * Next state function for the Authenticate packet
 * 
 * @param gensec_security GENSEC state
 * @param out_mem_ctx Memory context for *out
 * @param in The request, as a DATA_BLOB.  reply.data must be NULL
 * @param out The reply, as an allocated DATA_BLOB, caller to free.
 * @return Errors or NT_STATUS_OK if authentication sucessful
 */
NTSTATUS ntlmssp_server_auth(struct gensec_security *gensec_security, 
			     TALLOC_CTX *out_mem_ctx, 
			     const DATA_BLOB in, DATA_BLOB *out) ;

/** 
 * Return the credentials of a logged on user, including session keys
 * etc.
 *
 * Only valid after a successful authentication
 *
 * May only be called once per authentication.
 *
 */
NTSTATUS gensec_ntlmssp_session_info(struct gensec_security *gensec_security,
				     struct auth_session_info **session_info) ;

/**
 * Start NTLMSSP on the server side 
 *
 */
NTSTATUS gensec_ntlmssp_server_start(struct gensec_security *gensec_security);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __AUTH_NTLMSSP_PROTO_H__ */

